/*--------------------------------------------------------------------
 *
 *  Copyright (c) 2005-2025 by the GMT Team (https://www.generic-mapping-tools.org/team.html)
 *  See LICENSE.TXT file for copying and redistribution conditions.
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; version 3 or any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  Contact info: www.generic-mapping-tools.org
 *--------------------------------------------------------------------*/

/*!
 * \file mgd77_IGF_coeffs.h
 * \brief
 */

#define MGD77_IGF24_G0	978052.0
#define MGD77_IGF30_G0	978049.0
#define MGD77_IGF67_G0	978031.846
#define MGD77_IGF80_G0	978032.67714
#define MGD77_IGF24_G1	0.005285
#define MGD77_IGF30_G1	0.0052884
#define MGD77_IGF67_G1	0.0053024
#define MGD77_IGF80_G1	0.00193185138639
#define MGD77_IGF24_G2	7e-6
#define MGD77_IGF30_G2	0.0000059
#define MGD77_IGF67_G2	0.0000058
#define MGD77_IGF80_G2	0.00669437999013
#define MGD77_IGF24_G3	27e-6
