#
#	Definition file for inverted triangle symbol.  Use in psxy
#	as -Skinvtriangle/size.  Unlike -Si, this allows for
#	using patterns as fill
#
-0.433012701892	0.25	M
0.433012701892	0.25	D
0		-0.5	D
